/*
 * The API for an application wishing to use the parser in html.y
 *
 * 16 September 1997
 * Bert Bos
 * $Id: api.h,v 1.2 1998/11/26 13:23:12 bbos Exp $
 *
 * All char pointers are allocated on the heap, except for handle_error.
 * It is the handlers responsibility to eventually free them again,
 * the parser will not do so.
 *
 * start() may return a pointer that will then be passed back
 * by the parser all handler routines.
 */

/* handle_error -- called when a parse error occurred */
extern void handle_error(void *clientdata, const unsigned char *s, int lineno);

/* start -- called before the first event is reported */
extern void* start(void);

/* end -- called after the last event is reported */
extern void end(void *clientdata);

/* handle_comment -- called after a comment is parsed */
extern void handle_comment(void *clientdata, unsigned char *commenttext);

/* handle_text -- called after a tex chunk is parsed */
extern void handle_text(void *clientdata, unsigned char *text);

/* handle_declaration -- called after a declaration is parsed */
extern void handle_declaration(void *clientdata, unsigned char *decl);

/* handle_proc_instr -- called after a PI is parsed */
extern void handle_proc_instr(void *clientdata, unsigned char *pi_text);

/* handle_starttag -- called after a start tag is parsed */
extern void handle_starttag(void *clientdata, unsigned char *name, pairlist attribs);

/* handle_emptytag -- called after am empty tag is parsed */
extern void handle_emptytag(void *clientdata, unsigned char *name, pairlist attribs);

/* handle_pop -- called after an endtag is parsed (name may be "") */
extern void handle_endtag(void *clientdata, unsigned char *name);
