%{
  /* $Id: html.y,v 1.9 1998/12/05 00:52:45 bbos Exp $ */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "export.h"
#include "types.e"
#include "tree.e"
#include "api.h"

EXPORT extern int yyparse(void);
static void *data;

extern int yylex(void);
extern int yylineno;

static int nrerrors = 0;
#define MAX_ERRORS_REPORTED 20

/* yyerror -- report parse error */ 
static void yyerror(unsigned char *s)
{
  nrerrors++;
  if (nrerrors < MAX_ERRORS_REPORTED)
    handle_error(data, s, yylineno);
  else if (nrerrors == MAX_ERRORS_REPORTED)
    handle_error(data, "too many errors", yylineno);
  else
    ; /* don't report any more errors */
}

/* down -- convert a string to lowercase */
static inline void down(unsigned char *s)
{
  for (; *s; s++) *s = tolower(*s);
}

%}

%union {
    unsigned char *s;
    pairlist p;
}

%token <s> TEXT COMMENT START END NAME STRING DECL PROCINS
%token EMPTYEND

%type <p> attribute attributes

%%

start
  : {data = start();} document	{end(data);}
  ;
document
  : document COMMENT		{handle_comment(data, $2);}
  | document TEXT		{handle_text(data, $2);}
  | document starttag
  | document endtag
  | document DECL		{handle_declaration(data, $2);}
  | document PROCINS		{handle_proc_instr(data, $2);}
  | document error
  | /* empty */
  ;
starttag
  : START attributes '>'	{down($1); handle_starttag(data, $1, $2);}
  | START attributes EMPTYEND	{down($1); handle_emptytag(data, $1, $2);}
  ;
attributes
  : attribute attributes	{$$ = $1; $$->next = $2;}
  | /* empty */			{$$ = NULL;}
  ;
attribute
  : NAME			{pairlist h = malloc(sizeof(*h));
				 assert(h != NULL); h->name = $1;
				 down(h->name); h->value = NULL; $$ = h;}
  | NAME '=' NAME		{pairlist h = malloc(sizeof(*h));
				 assert(h != NULL); h->name = $1;
				 down(h->name); h->value = $3; $$ = h;}
  | NAME '=' STRING		{pairlist h = malloc(sizeof(*h));
				 assert(h != NULL); h->name = $1;
				 down(h->name); h->value = $3; $$ = h;}
  ;
endtag
  : END '>'			{down($1); handle_endtag(data, $1);}
  ;
