%option 8bit caseless nodefault noyywrap 
%option yylineno

%{
/* $Id: scan.l,v 1.12 1998/12/17 15:10:48 bbos Exp $ */
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "export.h"
#include "types.e"
#include "html.h"


EXPORT extern FILE *yyin;


/* esc -- remove outer quotes, escape '"', return malloc'ed string */
static unsigned char* esc(unsigned char* s)
{
  int i, j;
  unsigned char *u;

  /* Find new length */
  for (i = 0, j = 1; s[j] != s[0]; i++, j++) {
    if (s[j] == '"') i+= 4;
  }
  /* Copy and expand */
  u = malloc(i + 1);
  if (!u) {fprintf(stderr, "Out of memory\n"); exit(1);}
  for (i = 0, j = 1; s[j] != s[0]; i++, j++) {
    if (s[j] == '"')  {strcpy(u + i, "&#34;"); i += 4;}
    else u[i] = s[j];
  }
  u[i] = '\0';
  return u;
}
 
/* strndup -- allocate a string, copy n characters into it and add \0 */
static unsigned char* strndup(const unsigned char *s, size_t n)
{
  unsigned char *t = malloc(n + 1);
  if (!t) {fprintf(stderr, "Out of memory\n"); exit(1);}
  strncpy(t, s, n);
  t[n] = '\0';
  return t;
}

#if 0
/* append_utf8 -- append the UTF-8 sequence for code n */
static void append_utf8(unsigned char *t, unsigned int *i,
			 const unsigned int n)
{
  if (n <= 0x7F) {
    t[(*i)++] = n;
  } else if (n <= 0x7FF) {
    t[(*i)++] = 0xC0 | (n >> 6);
    t[(*i)++] = 0x80 | (n & 0x3F);
  } else if (n <= 0xFFFF) {
    t[(*i)++] = 0xE0 | (n >> 12);
    t[(*i)++] = 0x80 | ((n >> 6) & 0x3F);
    t[(*i)++] = 0x80 | (n & 0x3F);
  } else if (n <= 0x1FFFFF) {
    t[(*i)++] = 0xF0 | (n >> 18);
    t[(*i)++] = 0x80 | ((n >> 12) & 0x3F);
    t[(*i)++] = 0x80 | ((n >> 6) & 0x3F);
    t[(*i)++] = 0x80 | (n & 0x3F);
  } else if (n <= 0x3FFFFFF) {
    t[(*i)++] = 0xF0 | (n >> 24);
    t[(*i)++] = 0x80 | ((n >> 18) & 0x3F);
    t[(*i)++] = 0x80 | ((n >> 12) & 0x3F);
    t[(*i)++] = 0x80 | ((n >> 6) & 0x3F);
    t[(*i)++] = 0x80 | (n & 0x3F);
  } else {
    t[(*i)++] = 0xF0 | (n >> 30);
    t[(*i)++] = 0x80 | ((n >> 24) & 0x3F);
    t[(*i)++] = 0x80 | ((n >> 18) & 0x3F);
    t[(*i)++] = 0x80 | ((n >> 12) & 0x3F);
    t[(*i)++] = 0x80 | ((n >> 6) & 0x3F);
    t[(*i)++] = 0x80 | (n & 0x3F);
  }
}

/* append_named -- append the UTF-8 sequence of a named entity */
static void append_named(unsigned char *t, unsigned int *i,
			 const unsigned char *name, unsigned int len)
{
  const Entity *e = lookup_entity(name, len);
  unsigned int j;

  if (!e) {					/* Unknown entity, keep it */
    t[(*i)++] = '&';
    for (j = 0; j < len; j++, (*i)++) t[*i] = name[j];
    t[(*i)++] = ';';
  } else {					/* Convert to Unicode */
    append_utf8(t, i, e->code);
  }
}

/* exp -- expand entities to UTF-8 sequences (string is malloc'ed) */
static unsigned char* exp(const unsigned char *s, const int len)
{
  unsigned char *t;
  unsigned int i, j, n;

  /* Bug: we assume the new string isn't longer than the old one */
  t = malloc(len + 1);
  if (!t) {fprintf(stderr, "Out of memory\n"); exit(1);}
  for (i = 0, j = 0; i < len && j <= i;) {
    if (s[i] != '&') {				/* Literal character */
      t[j++] = s[i++];
    } else if (s[i+1] != '#') {			/* Named entity, eg. &eacute */
      i++;
      for (n = 0; isalnum(s[i+n]); n++) ;
      append_named(t, &j, s + i, n);
      i += n;
      if (s[i] == ';') i++;
    } else if (s[i+2] != 'x') {			/* Decimal entity, eg. &#70 */
      for (n = 0, i += 2; isdigit(s[i]); i++) n = 10 * n + s[i] - '0';
      append_utf8(t, &j, n);
      if (s[i] == ';') i++;
    } else {					/* Hex entity, eg. &#x5F */
      for (n = 0, i += 3; isxdigit(s[i]); i++)
	if (isdigit(s[i])) n = 16 * n + s[i] - '0';
	else n = 16 * n + toupper(s[i]) - 'A' + 10;
      append_utf8(t, &j, n);
      if (s[i] == ';') i++;
    }
    t[j] = '\0';
  }
  return t;
}
#endif /* 0 */

%}

/* thing is rather too permissive, but it will accept <img src=/path>... */

delim		[ \t\r\n\f="'<>]
nondelim	[^ \t\r\n\f="'<>]
name		[a-z0-9:.-]+
thing		{nondelim}+
comment		"<!--"([^-]+-)*"->"

%s MARKUP VALUE

%%


<INITIAL>"<"{name}	{BEGIN(MARKUP);yylval.s=strdup(yytext+1);return START;}
<INITIAL>"</"({name})?	{BEGIN(MARKUP);yylval.s=strdup(yytext+2);return END;}
<INITIAL>[^<]+		{yylval.s=strdup(yytext); return TEXT;}
<INITIAL>{comment}	{yylval.s=strndup(yytext+4,yyleng-7); return COMMENT;}
<INITIAL>"<!"[^>]*">"	{yylval.s=strndup(yytext+2,yyleng-3); return DECL;}
<INITIAL>"<?"[^>]*">"	{yylval.s=strndup(yytext+2,yyleng-3); return PROCINS;}

<MARKUP>{name}		{yylval.s = strdup(yytext); return NAME;}
<MARKUP>"="		{BEGIN(VALUE); return '=';}
<MARKUP>[ \t\r\n\f]+	{/* skip */}
<MARKUP>">"		{BEGIN(INITIAL); return '>';}
<MARKUP>"/>"		{BEGIN(INITIAL); return EMPTYEND;}

<VALUE>[ \t\r\n\f]+	{/* skip */}
<VALUE>{thing}		{BEGIN(MARKUP); yylval.s=strdup(yytext); return NAME;}
<VALUE>\"[^"]*\"	{BEGIN(MARKUP); yylval.s=esc(yytext); return STRING;}
<VALUE>\'[^']*\'	{BEGIN(MARKUP); yylval.s=esc(yytext); return STRING;}

.			{return *yytext; /* illegal char, in fact */}
