/*
 * Routines to wrap lines and indent them.
 *
 * Bert Bos
 * Created 10 May 1998
 * $Id: textwrap.c,v 1.5 1998/12/05 00:52:49 bbos Exp $
 */
#include <stdio.h>
#include <ctype.h>
#include <assert.h>
#include "export.h"
#include "types.e"
/*
#include "tree.e"
#include "html.tab.e"
#include "scan.e"
#include "api.h"
*/

#define MAXLINE 1024

static unsigned char buf[MAXLINE];
static int len = 0;
static int linelen = 0;
static int level = 0;
static int indent = 2;
static int maxlinelen = 72;
static Boolean need_space = False;
static Boolean need_indent = True;

/* indentline -- print spaces to required indentation */
static void indentline(void)
{
  int i;
  for (i = 0; i < indent * level; i++) putchar(' ');
  linelen = indent * level;
  need_indent = False;
  need_space = False;
}

/* set_indent -- set the amount of indent per level */
EXPORT void set_indent(int n) {indent = n;}

/* set_linelen -- set the maximum length of a line */
EXPORT void set_linelen(int n) {maxlinelen = n;}

/* flush -- print word in buf */
EXPORT void flush()
{
  if (len != 0) {
    if (need_indent) indentline();
    if (need_space) {
      if (linelen + len < maxlinelen) {putchar(' '); linelen++;}
      else {putchar('\n'); indentline();}
      need_space = False;
    }
    buf[len] = '\0';
    fputs(buf, stdout);
    linelen += len;
    len = 0;
  }
}

/* out -- add text to current output line, print line if getting too long */
EXPORT void out(unsigned char *s, Boolean preformatted)
{
  if (preformatted) {
    for (; *s; s++) {
      if (*s == '\n') {
	flush();
	putchar('\n');
	linelen = 0;
      } else {
	buf[len++] = *s;
      }
    }
  } else {
    for (; *s; s++) {
      if (!isspace(*s)) buf[len++] = *s;	/* Add to word */
      else {flush(); need_space = True;}	/* Start a new word */
    }
  }
}

/* outn -- add n chars to current output, print line if getting too long */
EXPORT void outn(unsigned char *s, size_t n, Boolean preformatted)
{
  size_t i;

  if (preformatted) {
    for (i = 0; i < n; i++) {
      if (s[i] == '\n') {
	flush();
	putchar('\n');
	linelen = 0;
      } else {
	buf[len++] = s[i];
      }
    }
  } else {
    for (i = 0; i < n; i++) {
      if (!isspace(s[i])) buf[len++] = s[i];	/* Add to word */
      else {flush(); need_space = True;}	/* Start a new word */
    }
  }
}

/* outln -- add string to output buffer, followed by '\n' */
EXPORT void outln(unsigned char *s, Boolean preformatted)
{
  if (s != NULL) out(s, preformatted);
  if (!preformatted) {
    flush();
    putchar('\n');
    need_indent = True;
  } else {
    out("\n", True);
  }
}

/* outc -- add one character to output buffer */
EXPORT void outc(unsigned char c, Boolean preformatted)
{
  if (preformatted) {
    if (c == '\n') {
      flush();
      putchar('\n');
      linelen = 0;
    } else {
      buf[len++] = c;
    }
  } else {
    if (!isspace(c)) buf[len++] = c;		/* Add to word */
    else {flush(); need_space = True;}		/* Start a new word */
  }
}

/* outbreak -- conditional new line; make sure next text starts on new line */
EXPORT void outbreak()
{
#if 1
  if (len != 0 || !need_indent) outln(NULL, False);
#else
  outln(NULL, False);
#endif
}

/* inc_indent -- increase indentation level by 1 */
EXPORT void inc_indent(void)
{
  flush();
  level++;
}

/* decc_indent -- decrease indentation level by 1 */
EXPORT void dec_indent(void)
{
  flush();
  level--;
}
